/*
 * $Id: EXAMPLE.c,v 0.2 1991/09/15 10:24:39 linus Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** EXAMPLE.c                             A small ISC subsystem demo program
**
** history:
** 910306 pen      initial coding
*/

#include <stdio.h>
#include <errno.h>

/*
** Could do a "#define ISC_UDGTYPE foobar" to change the type of
** the (IscSession) 'udg' member pointer type from "void *" to "foobar *".
*/
#include <isc.h>


extern void abort(void);


#define TIMEOUT    10000    /* 10 seconds */

Perror(char *msg)
{
  perror(msg);
  exit(1);
}

void
main(void)
{
  IscMaster   * mcb;   /* ISC master control block */
  IscSession  * scb;   /* ISC session control block */
  IscEvent    * ecb;   /* ISC event control block */



  /*
  ** Initialize the ISC subsystem, create a Master Control Block
  */
  if ((mcb = isc_initialize(NULL)) == NULL)
    Perror("isc_initialize");

  /*
  ** Establish a TCP session to a remote service (IRC)
  */
  if ((scb = isc_opentcp(mcb, "nanny.lysator.liu.se", "2000")) == NULL)
    Perror("isc_opentcp");

  /*
  ** Enable remote connection attempts at port #20000 and #20001
  */
  if (isc_listentcp(mcb, "20000") < 0)
    Perror("isc_listentcp-20000");

  if (isc_listentcp(mcb, "20001") < 0)
    Perror("isc_listentcp-20001");

  /*
  ** Handle all events
  */
  while (ecb = isc_getnextevent(mcb, TIMEOUT))
  {
    switch (ecb->event)
    {
      case ISC_EVENT_ERROR: /* ISC subsystem error */
	fprintf(stderr, "ISC-ERROR: errno = %d", errno);
	if (ecb->msg)
	  fprintf(stderr, " (%s)", ecb->msg->buffer);
	putc('\n', stderr);

	/*
	** Should probably do something more sensible than
	** just finishing off the session
	*/
	if (ecb->session)
	  isc_close(ecb->session);
	break;

      case ISC_EVENT_TIMEOUT: /* Timeout limit reached */
	puts("*** Timeout ***");
	break;

      case ISC_EVENT_LOGIN: /* New client requesting connection */
	printf("New session #%d on port %d connecting from: %s/%d\n, ",
	       ecb->session,
	       ecb->session->info.tcp.lport,
	       ecb->session->info.tcp.hostname,
	       ecb->session->info.tcp.rport);
	
	/*
	** Here we could assign the 'ecb->session->udg' member some
	** arbitrary value to be used later by our own routines
	*/
	break;

      case ISC_EVENT_LOGOUT: /* Client closed connection */
	if (ecb->session == scb)
	  printf("Server disconnecting\n");
	else
	  printf("Client #%d disconnecting\n", ecb->session);

	/*
	** Deallocate the storage reserved for the session
	*/
	isc_close(ecb->session);
	break;

      case ISC_EVENT_MESSAGE: /* Message from client or server */
	if (ecb->session == scb)
	  printf("Message from server: '%s'\n", ecb->msg->buffer);
	else
	{
	  printf("Message from client #%d: '%s'\n",
		 ecb->session,
		 ecb->msg->buffer);

	  /*
	  ** Reply with some text to the client
	  */
	  isc_write(ecb->session, "OK, got it\n\r", 12);
	  isc_flush(ecb->session);
	}
	break;

      default: /* NOTREACHED - OR SHOULDN'T BE ATLEAST :-) */
	Perror("NOTREACHED");
    }

    /*
    ** Finished with the event, lets get rid of it
    */
    isc_dispose(ecb);
  }

  /* NOTREACHED - I THINK ... */
  Perror("SHUTDOWN");
}
