/*
 * $Id: isc_event.c,v 0.5 1991/09/15 10:24:14 linus Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_event.c            		 definitions of ISC subsystem routines
**
** history:
**    900403 pen       initial coding.
**    900612 pen       rewrote the message buffering.
**    901102 pen       fixed bug in isc_gethostname().
**    910303 ceder     clean up. Removed everything that is lyskom-specific.
**    910304 pen       really removed everything lyskom-specific.. :-)
**    910910 ceder     fixed includefiles.
*/

static char *rcsid = "$Id: isc_event.c,v 0.5 1991/09/15 10:24:14 linus Exp $";

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#include <netdb.h>
#include <stddef.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <misc.h>
#include <unistd.h>

#if 0
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <netdb.h>
#include <stddef.h>
#include <string.h>
#include <fcntl.h>
#include <sys/file.h>
#endif

#include "isc.h"
#include "intern.h"



/* Get the local port number of the connection */
int
isc_getlportnum(int fd)
{
  struct sockaddr_in sin;
  int len;

  
  len = sizeof(sin);
  if (getsockname(fd, (struct sockaddr *) &sin, &len) < 0)
    return -1;

  return sin.sin_port;
}


/*
** Will return either the domain name or the IP number that the connection
** originates from. Returns NULL in case of an error.
*/
char *
isc_gethostinfo(int fd, int *port)
{
    static struct sockaddr_in  name;
    static struct hostent     *host;
    int                        len = sizeof(name);
  

    if (getpeername(fd, (struct sockaddr *) &name, &len) == -1)
	return NULL;

    if (port)
      *port = name.sin_port;
    
    host  = gethostbyaddr(&name.sin_addr,sizeof(struct in_addr),AF_INET);

    return isc_strdup((host && host->h_name[0]) ?
		      host->h_name : inet_ntoa(name.sin_addr));
}



IscMessage *
isc_readmsg(IscSession  * scb)
{
    int status;
    IscMessage *msg;


    msg = isc_allocmsg(scb->mcb->maxmsgsize);

    status = read(scb->fd, msg->buffer, msg->size);

    if (status > 0)
    {
	msg->length = status;
	msg->buffer[msg->length] = '\0';
	return msg;
    }
    else
    {
	isc_freemsg(msg);
	return NULL;
    }
}




/*
 * Wait for an event on a specific server.
 * Timeout is specified in milliseconds
 */
IscEvent *
isc_getnextevent(IscMaster  * mcb,
		 long         timeout)
{
    struct timeval    wait;
    fd_set            read_set;
#if 0    
    fd_set            write_set;
    fd_set            error_set;
#endif
    IscEvent        * isc_ei;
    IscEvent        * eip;
    IscSession      * scb;
    int               nfds;
    int               num_fds;
    IscMessage	    * msg;
    int               fd;
    int               lidx;
    

    isc_ei = (IscEvent *) isc_malloc(sizeof(IscEvent));

    isc_ei->msg     = NULL;
    isc_ei->session = NULL;
    
    /* Now go through the list of sessions checking for events */
    for (scb = mcb->sessions; scb != NULL; scb = scb->next)
    {
	if (isc_pollqueue(scb->rd_msg_q))
	{
	    /* Move to the next in the circular queue */
	    mcb->sessions = scb->next;

	    isc_ei->session = scb;
	    if (!(isc_ei->msg = isc_popqueue(scb->rd_msg_q)))
	    {
		/* Session terminated the connection */
		isc_ei->event  = ISC_EVENT_LOGOUT;

		/* Remove it from the list of active sessions */
		if (scb->next != scb)
		{
		    scb->prev->next = scb->next;
		    scb->next->prev = scb->prev;
		}
		else
		    mcb->sessions = NULL;

		scb->prev = NULL;
		scb->next = NULL;
		
		/* Note that you _must_ call isc_close()! */
	    }
	    else
		isc_ei->event  = ISC_EVENT_MESSAGE;

	    return isc_ei;
	}

	if (scb->next == mcb->sessions)
	    break;
    }
  
    /* Recursive call? Then we shouldn't get here really, but... */
    if (timeout == -1)
    {
      isc_dispose(isc_ei);
      return NULL;
    }
  
    /* Set up the list of file descriptors to wait on */
    FD_ZERO(&read_set);

    /* Add all listen-fd's */
    for (lidx = 0; lidx < mcb->listenc; lidx++)
        if (mcb->listenv[lidx].fd != -1)
	    FD_SET(mcb->listenv[lidx].fd, &read_set);

    /* Fill in the fd_set */
    num_fds = 0;
    for (scb = mcb->sessions; scb != NULL; scb = scb->next)
    {
        isc_flush(scb);
      
	FD_SET(scb->fd, &read_set);
	++num_fds;

	if (scb->next == mcb->sessions)
	    break;
    }

    /* Set up the timeout structure */
    wait.tv_sec  = timeout / 1000;
    wait.tv_usec = (timeout * 1000L) % 1000000L;

    /* Wait for event */
    nfds = select(FD_SETSIZE,
		  &read_set,
		  (fd_set *) NULL,
		  (fd_set *) NULL,
		  &wait);

    /* Error in select? */
    if (nfds < 0)
    {
	isc_ei->event   = ISC_EVENT_ERROR;
	isc_ei->session = NULL;
	isc_ei->msg     = isc_mkstrmsg("Error in select()");
	return isc_ei;
    }
  
    /* Timeout? */
    if (nfds == 0)
    {
	isc_ei->event   = ISC_EVENT_TIMEOUT;
	isc_ei->session = NULL;
	return isc_ei;
    }
  
    /* New connection? */
    for (lidx = 0; lidx < mcb->listenc; lidx++)
      if (mcb->listenv[lidx].fd != -1 &&
	  FD_ISSET(mcb->listenv[lidx].fd, &read_set))
      {
	if ((fd = accept(mcb->listenv[lidx].fd,
			 (struct sockaddr *) NULL,
			 (int *) NULL)) < 0)
	{
	    /* Strange - that didn't work */
	    isc_ei->event   = ISC_EVENT_ERROR;
	    isc_ei->session = NULL;
	    isc_ei->msg     = isc_mkstrmsg("Failed accept() in new session");
	    return isc_ei;
	}

	scb = isc_openfd(mcb, fd);
	if (!scb)
	{
	    isc_ei->event   = ISC_EVENT_ERROR;
	    isc_ei->session = NULL;
	    isc_ei->msg     = isc_mkstrmsg("Failed isc_openfd() "
					   "in new session");
	    isc_free(scb);
	    return isc_ei;
	}

	/* Fill in the session info structure */
	scb->type = mcb->listenv[lidx].type;
	switch (scb->type)
	{
	  case ISC_TYPE_TCP:
  	    scb->info.tcp.listen   = lidx;
	    scb->info.tcp.hostname = isc_gethostinfo(scb->fd,
						     &scb->info.tcp.rport);
	    scb->info.tcp.lport    = isc_getlportnum(scb->fd);
	    break;

	  case ISC_TYPE_PIPE:
  	    scb->info.pipe.listen   = lidx;
	    scb->info.pipe.pathname = (char *) mcb->listenv[lidx].info;
	    break;

	  case ISC_TYPE_FILE:
	  case ISC_TYPE_UNKNOWN:
	    isc_log("isc_getnextevent(): Shouldn't happen: listen-type");
	}
	
	/* Fill in the event info structure */
	isc_ei->session  = scb;
	isc_ei->event    = ISC_EVENT_LOGIN;
      
	return isc_ei;	    
    }

  
    /* Check for messages from sessions */
    for (scb = mcb->sessions; scb != NULL; scb = scb->next)
    {
	if (FD_ISSET(scb->fd, &read_set))
	{
	    msg = isc_readmsg(scb);
	    if (msg)
	    {
		scb->stats.rx.bytes += msg->length;
		scb->stats.rx.packets++;
	    }
	    isc_pushqueue(scb->rd_msg_q, msg);
        }

	if (scb->next == mcb->sessions)
	    break;
    }

    /* Call ourself one more time - this might have to be rewritten */
    eip = isc_getnextevent(mcb, -1);
    if (eip)
    {
        isc_dispose(isc_ei);
	return eip;
    }
  
    /* Strange - select returned, yet no fd were found to be active */
    isc_ei->event   = ISC_EVENT_ERROR;
    isc_ei->session = NULL;
    isc_ei->msg     = isc_mkstrmsg("Stale ISC_EVENT_ERROR");
    
    return isc_ei;
}




void
isc_dispose(IscEvent *ep)
{
  if (!ep)
    return;

  if (ep->msg)
    isc_freemsg(ep->msg);

  isc_free(ep);
}
