/*
 * $Id: isc_listen.c,v 0.5 1991/09/15 10:24:10 linus Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_listen.c                                         Handle "listen" ports
*/

static char *rcsid = "$Id: isc_listen.c,v 0.5 1991/09/15 10:24:10 linus Exp $";

#include <stddef.h>
#include <stdlib.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <sys/file.h>
#include <misc.h>
#include <unistd.h>

#include "isc.h"
#include "intern.h"




static int
isc_addlisten( IscMaster        * mcb,
 	       IscSessionType     type,
	       int                fd,
	       void             * info )
{
  int i;


  for (i = 0; i < mcb->listenc; i++)
    if (mcb->listenv[i].fd == -1)
      break;

  if (i == mcb->listenc)
  {
    if (mcb->listenc == 0)
      mcb->listenv = (struct isc_listen *)
	isc_malloc(sizeof(struct isc_listen));
    else
      mcb->listenv = (struct isc_listen *)
	isc_realloc(mcb->listenv,
		    sizeof(struct isc_listen)*(mcb->listenc+1));
    mcb->listenc++;
  }

  mcb->listenv[i].type = type;
  mcb->listenv[i].fd   = fd;
  mcb->listenv[i].info = info;
    
  return i;
}



/*
 * Will open a new port and bind a stream socket to it. Returns -1 in case of
 * an error, or a FD to listen at for new connections.
 */
static int
isc_openport(int port,  int backlog,  int maxretries)
{
  struct sockaddr_in  server;
  int 		      fd;
  int		      retries;
  

  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    return -1;

  server.sin_family      = AF_INET;
  server.sin_addr.s_addr = INADDR_ANY;
  server.sin_port 	 = htons(port);
  
  if (setsockopt(fd, SOL_SOCKET, SO_DONTLINGER, 0, 0))
    isc_log("setsockopt(SOL_SOCKET, SO_DONTLINGER, 0, 0)");

  if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, 0, 0))
    isc_log("setsockopt(SOL_SOCKET, SO_REUSERADDR, 0, 0)");
    
  for (retries = 0; retries < maxretries; sleep(1), retries++)
  {
    if (bind(fd, (struct sockaddr *) &server, sizeof(server)) != -1)
      break;
  }

  if (retries >= maxretries)
  {
    close(fd);
    return -1;
  }

  if (listen(fd, backlog) == -1)
  {
    close(fd);
    return -1;
  }

  return fd;
}




/*
** Establish a port to listen at for new connections
*/
int
isc_listentcp(IscMaster   * mcb,
	      const char  * service)
{
  int fd;
  int port;
  struct servent *sp;

    
  if ((sp = getservbyname(service, "tcp")) == NULL)
    port = atoi(service);
  else
    port = ntohs(sp->s_port);

  if ((fd = isc_openport(port, SOMAXCONN, 10)) == -1)
    return -1;

  return isc_addlisten(mcb, ISC_TYPE_TCP, fd, (void *) isc_strdup(service));
}


/*
** Stop to listen on a specific port
*/
int
isc_unlisten(IscMaster  * mcb,
	     int          index)
{
  if (index < 0 || index >= mcb->listenc || mcb->listenv[index].fd == -1)
    return -1;
    
  close(mcb->listenv[index].fd);

  mcb->listenv[index].type = ISC_TYPE_UNKNOWN;
  mcb->listenv[index].fd   = -1;
  if (mcb->listenv[index].info)
  {
    isc_free(mcb->listenv[index].info);
    mcb->listenv[index].info = NULL;
  }
  

  return index;
}
