/*
 * $Id: isc_message.c,v 0.5 1991/09/21 12:37:21 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_message.c                      Generic "IscMessage" handling functions
**
** history:
** 910305 pen      moved into separate file
*/

static char *rcsid = "$Id: isc_message.c,v 0.5 1991/09/21 12:37:21 ceder Exp $";

#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include "isc.h"
#include "isc-dump.h"
#include "intern.h"


static long allocated_messages = 0;

IscMessage *
isc_allocmsg(size_t size)
{
    IscMessage  * msg;

    
    msg = (IscMessage *) isc_malloc(sizeof(IscMessage));
    msg->buffer = (char *) isc_malloc(size+1);
    msg->size   = size;
    msg->length = 0;

    ++allocated_messages;

    return msg;
}


IscMessage *
isc_reallocmsg(IscMessage  * msg,
	       size_t        newsize)
{
    msg->buffer = (char *) isc_realloc(msg->buffer, newsize+1);
    msg->size   = newsize;
    return msg;
}



void
isc_freemsg(IscMessage  * msg)
{
    isc_free(msg->buffer);
    isc_free(msg);

    --allocated_messages;
}



IscMessage *
isc_mkstrmsg(const char *str)
{
  IscMessage  * msg;

  
  msg = isc_allocmsg(strlen(str)+1);
  strcpy(msg->buffer,str);
  msg->length = strlen(str);

  return msg;
}

void
dump_isc_message_counts(FILE *stat_file)
{
    fprintf(stat_file, "---" __FILE__ ":\n"
	    "\tAllocated isc_messages:    %d\n",
	    allocated_messages);
}
