/*
 * $Id: isc_output.c,v 0.5 1991/09/21 12:07:53 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_output.c                                   Isc data output functions
**
** history:
** 910305 pen      moved into separate file
** 910310 pen      added isc_send()
** 910910 ceder    added include file <unistd.h> for declaration of write.
*/

static char *rcsid = "$Id: isc_output.c,v 0.5 1991/09/21 12:07:53 ceder Exp $";

#include <stddef.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/file.h>
#include <errno.h>
#include <unistd.h>
#include "isc.h"
#include "intern.h"



void
isc_flush(IscSession  * scb)
{
  IscMessage  * msg;
  int           failed;
  int           wlen;
  int           loopcnt;
  
  
  /* Data in the block buffer? */
  if (scb->sendindex > 0)
  {
    /* Too many queued blocks? */
    if (isc_sizequeue(scb->wr_msg_q) >= scb->mcb->maxqueuedsize)
    {
      /* Well, should do something here now... */
      if (!scb->kill_him)
	isc_log("isc_flush: session transmit buffer overflow (%d >= %d).\n",
	    isc_sizequeue(scb->wr_msg_q), scb->mcb->maxqueuedsize);

      scb->kill_him  = 1; /* Lets kill this session off */
      scb->sendindex = 0; /* And lets fake a write */
      return;
    }
    
    msg = isc_allocmsg(scb->sendindex);
    memcpy(msg->buffer, scb->sendbuf, scb->sendindex);
    msg->length = scb->sendindex;
    isc_pushqueue(scb->wr_msg_q, msg);
    scb->sendindex = 0;
  }

  /* Queued entries? Send as much as possible */
  failed = 0;
  loopcnt = 0;
  while ((msg = isc_topqueue(scb->wr_msg_q)) != NULL &&
	 !failed && loopcnt < scb->mcb->maxdequeuelen)
  {
    wlen = write(scb->fd, msg->buffer, msg->length);

    if (wlen < 0)
    {
      switch ( errno )
      {
        case EWOULDBLOCK:
	  wlen = 0;
	  break;

        case EPIPE:
	  scb->kill_him = 1;
	  return;

        default:
	  if (!scb->kill_him)
	    isc_log("isc_flush(): write(%d, ..., %d) failed: error = %s\n",
		scb->fd, msg->length, strerror(errno));
	  scb->kill_him  = 1;
          scb->sendindex = 0;
	  return;
      }
    }

    if (wlen > 0)
    {
      scb->stats.tx.bytes += wlen;
      scb->stats.tx.packets++;
    }
      
    msg->length -= wlen;
    if (msg->length > 0)
    {
      /* write() could not transmit the whole buffer */
      failed = 1;
      if (wlen > 0)
	memcpy(msg->buffer, msg->buffer + wlen, msg->length);
    }
    else
    {
      /* Drop the topmost entry */
      msg = isc_popqueue(scb->wr_msg_q);
      isc_freemsg(msg);
    }

    loopcnt++;
  }
}



void
isc_send(IscSession  *scb, IscMessage  *msg)
{
  /* Must flush if pending "stdout" output */
  if (scb->sendindex)
    isc_flush(scb);

  isc_pushqueue(scb->wr_msg_q, msg);
}
