/*
 * $Id: isc_queue.c,v 0.3 1991/09/15 10:23:52 linus Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_queue.c                           Handle queues for the ISC subsystem
**
** history:
** 910305 pen      moved to separate file
*/

static char *rcsid = "$Id: isc_queue.c,v 0.3 1991/09/15 10:23:52 linus Exp $";

#include <stddef.h>
#include "isc.h"
#include "intern.h"


IscMsgQueue *
isc_newqueue(void)
{
    IscMsgQueue  * msg_q;

    
    msg_q = (IscMsgQueue *) isc_malloc(sizeof(IscMsgQueue));
    msg_q->head = NULL;
    msg_q->tail = NULL;
    msg_q->entries = 0;
    
    return msg_q;
}


/* Return TRUE if the queue is not empty. */
int
isc_pollqueue(IscMsgQueue  * queue)
{
    return queue->head != NULL;
}


void
isc_killqueue(IscMsgQueue  * queue)
{
    IscMsgQE  * mqe;
    IscMsgQE  * prev;

    
    if ( queue->head != NULL )
    {
	mqe = queue->head;
	while ( mqe != NULL )
	{
	    prev = mqe;
	    mqe = mqe->next;
	    isc_free(prev);
	}
    }

    isc_free(queue);
    return;
}


void
isc_pushqueue(IscMsgQueue  * queue,
	      IscMessage   * msg)
{
  IscMsgQE  * mqe;

  
  mqe = (IscMsgQE *) isc_malloc(sizeof(IscMsgQE));

  mqe->msg  = msg;
  mqe->prev = queue->tail;
  mqe->next = NULL;

  if (queue->head == NULL)
    queue->head = mqe;
  
  if (queue->tail)
    queue->tail->next = mqe;

  queue->tail = mqe;
  queue->entries++;
}


IscMessage *
isc_topqueue(IscMsgQueue  * queue)
{
    if (queue != NULL && queue->head != NULL)
        return queue->head->msg;
    else
        return NULL;
}


int
isc_sizequeue(IscMsgQueue  * queue)
{
  if (queue)
    return queue->entries;
  else
    return -1;
}
	      

IscMessage *
isc_popqueue(IscMsgQueue  * queue)
{
    IscMsgQE    * mqe;
    IscMessage  * msg;

    
    if (queue != NULL && queue->head != NULL)
    {
	mqe = queue->head;
	msg = mqe->msg;
	queue->head = mqe->next;

	if (queue->head)
	    queue->head->prev = NULL;
	else
	    queue->tail = NULL;

	queue->entries--;
	isc_free(mqe);
	return msg;
    }
    else
	return NULL;
}

