/*
 * $Id: isc_stdout.c,v 0.3 1991/09/15 10:23:44 linus Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_stdout.c                 Some nice-to-have functions for output..
**
** history:
** 910305 pen      moved into separate file
*/

static char *rcsid = "$Id: isc_stdout.c,v 0.3 1991/09/15 10:23:44 linus Exp $";

#include <stdarg.h>
#include <string.h>
#include "isc.h"


extern int _printf( int (*fctn)(int chr), const char *format, ...);
  

int
isc_putc(int           chr,
	 IscSession  * scb)
{
  while (scb->sendindex == sizeof(scb->sendbuf))
    isc_flush(scb);

  return scb->sendbuf[scb->sendindex++] = chr;
}



int
isc_write(IscSession  * scb,
	  const void  * buffer,
	  size_t        length)
{
  const char  * bp;
  int           len;
  int           blen;
  int           clen;


  bp   = (const char *) buffer;
  clen = length;
  
  while (clen > 0)
  {
    blen = sizeof(scb->sendbuf) - scb->sendindex;

    /* Make room in sendbuf */
    while (blen == 0)
    {
      isc_flush(scb);
      blen = sizeof(scb->sendbuf) - scb->sendindex;
    }

    len  = (clen > blen ? blen : clen);
    
    memcpy(scb->sendbuf+scb->sendindex, bp, len);
    scb->sendindex += len;
    clen -= len;
    bp   += len;
  }

  return length;
}



static IscSession  * send_scb;  /* Used by isc_printf */

static int send_putc(int c)
{
  return isc_putc(c, send_scb);
}


int
isc_vprintf(IscSession  * scb,
	    const char  * format,
	    va_list       AP)
{
  send_scb = scb;
  
  return _printf(send_putc, format, AP);
}


int
isc_printf(IscSession  * scb,
	   const char  * format,
	   ...)
{
  va_list AP;
  int cnt;

  
  va_start(AP, format);

  send_scb = scb;
  
  cnt = _printf(send_putc, format, AP);

  va_end(AP);

  return cnt;
}
